;
; 512 colour test
;

_xres	.equ 256	; set this before including 'startup.asm'
			; that will set up a 320x240 screen in place
			; of the default 256x240 screen

	.include "startup.asm"
	.nomlist
	.list

; ----
; defines

NB_PICS	.equ 1		; the number of pictures in the slideshow

; ----
; variables

	.zp
pic:	.rs  1		; the current picture number
scanline:	.ds 2	; the line number for the scanline interrupt
sinevalue: .ds 2	;Some temporary val for the sine wave.
scroll1_add: .ds 2		;A value for scrolling the first plane.

	; wavy scroll

wave_base:	.ds 1	; base index for the wave table
wave_idx:	.ds 1	; index in the wave table
wave_delay:	.ds 1   ; delay for wave scrolling

colour_val:	.ds 2	;The value for the colour
colour_val2:	.ds 2	;The value for the colour
colour_index:	.ds 2	;The colour index
cmpval: .ds 1		;compare value


;[ MAIN CODE ]

	.code
	.bank MAIN_BANK
	.org  $C000
main:
	; disable display

	vsync
	vreg  #5
	lda   #$0C
	sta   $0002

	; load the picture in VRAM

	jsr	load_pic

	stw	#$0100,$0402	;Set sprite index
	stwz	$0404

	; enable display

	vsync
	vreg  #5
	lda   #$CC
	sta   $0002

	vsync

	lda	#$60
	sta	cmpval

	setvec #VSYNC,vsync_proc
	setvec #HSYNC,hsync_proc
	vec_on #VSYNC
	vec_on #HSYNC

.waitstate

	jmp   .waitstate

;**************************************************
;------ Interrupt code --------------------------

; ----
; vsync_proc
; ----
; called every VSYNC
; ----

vsync_proc:
	st0   #6		; restart the scanline counter on the first
	stw   #$40,video_data	; line
	stw   #$40,<scanline

	stwz	colour_val
	stwz	colour_index
	stw	colour_index,$0402	;Set pal index
	stw	colour_val,$0404
	
	stw	#$08,colour_index

	tst	#$20,joytrg
	beq	.NoDecCmpVal		;Use the joypad left/right for fine tuning.
	dec	cmpval
.NoDecCmpVal
	tst	#$80,joytrg
	beq	.NoIncCmpVal
	inc	cmpval
.NoIncCmpVal

;---------------------------
; enable display
;	vreg  #5
;	lda   #$CC
;	sta   $0002
;----------------------
	rts

; ----
; hsync_proc
; ----
; called every HSYNC
; ----

hsync_proc:



	lda	<scanline		;Start at the right scanline
	cmp	cmpval
	bcs	.StartPalChange
	jmp	.NoStartPalChange
	cmp	#$F8
	bcc	.MorePalChange
	jmp	.NoMorePalChange
.StartPalChange
.MorePalChange

	stw	colour_index,$0402	;Set pal index
	stw	colour_val,$0404
	inc	colour_index
	incw	colour_val

	stw	colour_index,$0402	;Set pal index
	stw	colour_val,$0404	;Write 2 palettes per scanline
	incw	colour_val
	inc	colour_index

	stw	colour_index,$0402	;Set pal index
	stw	colour_val,$0404	;Write 3 palettes per scanline
	incw	colour_val
	inc	colour_index

	stw	colour_index,$0402	;Set pal index
	stw	colour_val,$0404	;Write 4 palettes per scanline
	incw	colour_val
	inc	colour_index

	lda	colour_index
	ora	#$08		;Make sure it's 08-0F or 18-1F
	and	#$7F
	sta	colour_index



.NoMorePalChange

.NoStartPalChange

	
.update:

	; reprogram the scanline counter for the next line

.scanline:
	st0   #6
	incw  <scanline
	stw   <scanline,video_data

	rts

;[ SUB CODE ]


; ----
; load_pic
; ----
; upload a whole pic in VRAM
; ----

load_pic:
	lda   <pic		; get the current pic number

	asl   A			; mul it by 6 (each picture takes six banks,
	sta   <_al		; five for the graphics and one for the BAT
	asl   A			; and the palette)
	add   <_al

	add   #2		; add 2 - our pictures are stored starting
				; from bank 2

	sta   <_ah		; store the bank index

	; load the graphic in VRAM, one bank at a time
	
	lda   <_ah		; first bank
	tam   #4
	vload $1000,$8000,#$1000
	inc   <_ah		; second bank
	lda   <_ah
	tam   #4
	vload $2000,$8000,#$1000
	inc   <_ah		; third bank
	lda   <_ah
	tam   #4
	vload $3000,$8000,#$1000
	inc   <_ah		; fourth bank
	lda   <_ah
	tam   #4
	vload $4000,$8000,#$1000

	; vsync before setting the palette, to avoid snow

	vsync

	; now set the palette

	inc   <_ah
	lda   <_ah
	tam   #4
	set_bgpal #0,$8000,#16

	; and finaly copy the BAT

	batcpy $0,\
	       #64,\
	       $8200,\
	       #32,\
	       #32

	; ok, done

	rts


;[ PICTURE DATA ]

	.data

	; pictures can contain up to 256 colors, but as you know the
	; PC-Engine can display only 16 colors per 8x8 tile, so be careful
	; when making pictures. And also take care of the color alignment,
	; a tile can use, for example, color 0 to 15, but not color 4 plus
	; color 26 plus color 125, all the colors used in a tile must
	; belong to the same sub-palette (colors 0 - 15, 16 - 31, etc...)
        ;
	; if you have problems making such pictures, use only 16 colors
	; PCX files, you will have less problems

PIC_BANK .func (((\1)*5)+MAIN_BANK+1)	; a little function to help
					; referencing banks

PIC	.macro			; hmm a macro could be handy too...
	 .bank PIC_BANK(\1)
	 .org  $8000
	 .incchr \2,32,32
	 .bank PIC_BANK(\1)+4
	 .org  $8000
	 .incpal \2
	 .org  $8200
	 .incbat \2,$1000,32,32
	.endm

	PIC 0,"512test3.pcx"	; what would we do without macros? :)

	; etc... up to 21 pictures.
	;
	; at your turn now, make us some cool slideshow!

